@@ -1,6 +1,6 @@
 #new predict.segmented
 predict.segmented<-function(object, newdata, se.fit=FALSE, interval=c("none","confidence", "prediction"), 
-            type = c("link", "response"),# "terms"),
+            type = c("link", "response"), na.action=na.omit,# "terms"),
             level=0.95, .coef=NULL, ...){
   blockdiag <- function(...) {
     args <- list(...)
@@ -109,13 +109,21 @@ predict.segmented<-function(object, newdata, se.fit=FALSE, interval=c("none","co
   
   if(missing(newdata)){
     X <- model.matrix(object)
+    idNA<- rep(FALSE, nrow(X))
   } else {
     #browser()
     #nomiLin <- setdiff(all.vars(formula(object))[-1], c(object$nameUV$U,object$nameUV$V))
     nomiLin <- setdiff(all.vars(as.formula(paste("~",paste(formula(object))[3]))), c(object$nameUV$U,object$nameUV$V))
     if(any(is.na(match(nomiLin, names(newdata))))) stop(" 'newdata' should includes all variables")
     #devi trasformare la variabili segmented attraverso dummy.matrix()
-    #browser()
+    
+    na.arg <- deparse(substitute(na.action))
+    idNA<- !complete.cases(newdata)
+    if(any(idNA)){
+      newdata<-na.omit(newdata)
+    }
+    if(!na.arg%in%c("na.omit","na.pass")) stop("na.action should be 'na.omit' or 'na.pass'")
+    
     n<-nrow(newdata)
     r<-NULL
     if(length(object$call$obj)>0){ #se l'ogg e' stato ottenuto da segmented.*
@@ -138,6 +146,9 @@ predict.segmented<-function(object, newdata, se.fit=FALSE, interval=c("none","co
               contrasts=object$contrasts, xlev = object$xlevels)
       
     } else { #se l'ogg e' stato ottenuto da segreg
+      #browser()
+      
+      
       Fo<-as.formula(object$nameUV$formulaSegAllTerms)
       if(any(all.vars(Fo)%in%names(object$xlevels))){
         M<-model.matrix(Fo, data=newdata, 
@@ -204,8 +215,6 @@ predict.segmented<-function(object, newdata, se.fit=FALSE, interval=c("none","co
   
   mu <- eta<- drop(X.noV%*% estcoef.noV)
   
-  #ATTENZIONE c'e' il problema dell'appaiamento dei nomi!!!
-  #il problema e' che estcoef non ha sempre nomi!! 
 
   X <- X[,c(colnomi.noV, nameV),drop=FALSE]
 
@@ -232,12 +241,27 @@ predict.segmented<-function(object, newdata, se.fit=FALSE, interval=c("none","co
       z <- abs(qt((1-level)/2, df=object$df.residual))
       s2<- summary(object)$sigma^2
     }
+    
+    if(any(idNA) && na.arg=="na.pass"){
+      mu0<-mu
+      se0<-se
+      mu<-se<- rep(NA, length(idNA))
+      mu[!idNA]<-mu0
+      se[!idNA]<-se0
+    }
+    
     if(interval=="confidence"){
       mu<-cbind(fit=mu, lwr=mu-z*se, upr=mu+z*se)
     }
     if(interval=="prediction"){
       mu<-cbind(fit=mu, lwr=mu-z*sqrt(se^2+s2), upr=mu+z*sqrt(se^2+s2))
     }
+  } else {
+    if(any(idNA)&& na.arg=="na.pass"){
+      mu0<-mu
+      mu<- rep(NA, length(idNA))
+      mu[!idNA]<-mu0
+    }
   }
   if(se.fit) {
     mu <- list(fit=mu, se.fit=se, df= object$df.residual, residual.scale=sqrt(s2))
