@@ -112,6 +112,7 @@ seg.def.fit.boot<-function(obj, Z, PSI, mfExt, opz, n.boot=10, size.boot=NULL, j
             all.est.psi.boot[k,]<-est.psi.boot<-o.boot$psi
             } else {
             est.psi.boot<-apply(limZ,2,function(r)runif(1,r[1],r[2]))
+            est.psi.boot<- unlist(tapply(est.psi.boot, opz$id.psi.group, sort))
             }
             PSI <- matrix(est.psi.boot, n, ncol = length(est.psi.boot), byrow=TRUE)
             #opz$h<-max(opz$h*.9, .2)
@@ -158,33 +159,55 @@ seg.def.fit.boot<-function(obj, Z, PSI, mfExt, opz, n.boot=10, size.boot=NULL, j
         #quando vengono restituiti psi troppo vicini e l'SE non si puo' calcolare, possiamo distanziarli..
         #Pero' il processo deve essere esteso nel caso in cui ci sono 3 psi vicini..
         min.n <- opz$min.n-1
+        if(min.n>1){
+          min1<- function(x, k=min.n-1){
+            for(i in 1:k) x<-x[-which.min(x)]
+            min(x)
+          }
+          max1<-function(x,k=min.n-1){
+            for(i in 1:k) x<-x[-which.max(x)]
+            max(x)
+          }
+        } else {
+          min1<-min
+          max1<-max
+        }
         npsi <- tapply(opz$id.psi.group, opz$id.psi.group, length)
         nomiAll <- colnames(rangeZ) #rep(opz$nomiSeg, npsi)
         nomiSeg <- unique(nomiAll)
         newPsi<-vector("list", length(npsi) )
         for(.j in 1:length(npsi)){
-          psi.j <- sort(est.psi0[opz$id.psi.group==.j])
+          psi.j <- sort(est.psi0[opz$id.psi.group==.j]) #psi della stessa variabile segmented
           id  <- nomiSeg[.j]==nomiAll
           Z.ok <- unique(Z[, id, drop=FALSE][,1])
           m.j <- min(limZ[1,id])
           M.j <- max(limZ[2,id])
-          h=1/1.05
-          id.while<-tabulate(cut(Z.ok, c(m.j-10, psi.j, M.j+10), labels=FALSE))<=min.n
-          while(any(id.while)){
-            h<-h*1.05
-            ll <- min(diff(sort(unique(Z.ok))))*h
-            M <- matrix(c(m.j, rep(psi.j, each=2), M.j), ncol=2, byrow=TRUE)
-            Delta <- diff(c(m.j, psi.j, M.j))
-            #id.row <- which.min(Delta)
-            id.row <- which(id.while)
-            id.row <- id.row[which.min(Delta[id.row])]
-            if(id.row<length(Delta)){ #increnmta psi se il "problema" NON riguarda l'ultimo psi
-              psi.j[id.row]<- psi.j[id.row] + abs(min(Delta)-ll)/2
-            } else { #.. altrimenti riducilo
-              psi.j[length(psi.j)]<- psi.j[length(psi.j)] - abs(min(Delta)-ll)/2
-            }
-            id.while<-tabulate(cut(Z.ok, c(m.j-10, psi.j, M.j+10), labels=FALSE))<=min.n
+          #h=1/1.05
+          for(.k in 1:length(psi.j)){
+            id.group<-cut(Z.ok, c(m.j-10^8, psi.j, M.j+10^8), labels=FALSE)
+            n.j<-tabulate(id.group)#<=min.n
+            #per ogni psi calcola il min e il max dei segmenti prima e dopo psi. 
+            #se questi segmenti hanno min.n osservazioni considera u min e max fittizzi per evitare che il nuovo psi
+            #modificato porti a segmenti con bassa numerosita'..
+            M.j.k<- if(n.j[.k]>0) max1(Z.ok[id.group==.k])  -10^6*(n.j[.k]<=min.n) else -10^6*(n.j[.k]<=min.n) 
+            m.j.k<- if(n.j[.k+1]>0) min1(Z.ok[id.group==.k+1])+10^6*(n.j[.k+1]<=min.n) else  10^6*(n.j[.k]<=min.n)
+            psi.j[.k]<- psi.j[.k] + ifelse(abs(M.j.k-psi.j[.k])<abs(m.j.k-psi.j[.k]), M.j.k-psi.j[.k]-.0001, m.j.k-psi.j[.k]+.0001  )
           }
+          # while(any(id.while)){
+          #   h<-h*1.05
+          #   ll <- min(diff(sort(unique(Z.ok))))*h
+          #   M <- matrix(c(m.j, rep(psi.j, each=2), M.j), ncol=2, byrow=TRUE)
+          #   Delta <- diff(c(m.j, psi.j, M.j))
+          #   #id.row <- which.min(Delta)
+          #   id.row <- which(id.while)
+          #   id.row <- id.row[which.min(Delta[id.row])]
+          #   if(id.row<length(Delta)){ #increnmta psi se il "problema" NON riguarda l'ultimo psi
+          #     psi.j[id.row]<- psi.j[id.row] + abs(min(Delta)-ll)/2
+          #   } else { #.. altrimenti riducilo
+          #     psi.j[length(psi.j)]<- psi.j[length(psi.j)] - abs(min(Delta)-ll)/2
+          #   }
+          #   id.while<-tabulate(cut(Z.ok, c(m.j-10, psi.j, M.j+10), labels=FALSE))<=min.n
+          # }
           newPsi[[.j]]<-psi.j
         } #end .j
         est.psi0 <- unlist(newPsi)
@@ -195,11 +218,8 @@ seg.def.fit.boot<-function(obj, Z, PSI, mfExt, opz, n.boot=10, size.boot=NULL, j
         #warning("'Convergence' is suspect: the final fit could be unreliable. Try to re-run by increasing 'break.boot'", call.=FALSE, immediate.=TRUE)
       }
       
-      # if(is.null(o0$obj)){
-      #   PSI1 <- matrix(est.psi0, n, ncol = length(est.psi0), byrow=TRUE)
-      #   o0 <- try(suppressWarnings(seg.def.fit(obj, Z, PSI1, mfExt, opz1)), silent=TRUE)
-      #   warning("The final fit can be unreliable (possibly mispecified segmented relationship)", call.=FALSE, immediate.=TRUE)
-      # }
+      
+      
       if(!is.list(o0)) return(0)
       o0$boot.restart<-ris
       o0$seed<-seed
